@@ -112,7 +112,7 @@ class Storage(driver.Base):
         mpath = self.create_manta_path(path)
 
         try:
-            return self.client.get_object(mpath)
+            return self.get_object(mpath)
         except Exception:
             raise exceptions.FileNotFoundError("File %s not found" % path)
 
@@ -130,9 +130,36 @@ class Storage(driver.Base):
         res, content = self.client._request(path=mpath, method='HEAD')
         return str(res.status) != '404'
 
+    def get_object(self, mpath, accept="*/*"):
+        headers = {
+            "cache-control": "no-cache",
+            "Accept": accept
+        }
+
+        res, content = self.client._request(mpath, "GET", headers=headers)
+
+        if res["status"] not in ("200", "304"):
+            raise errors.MantaAPIError(res, content)
+
+        if len(content) != int(res["content-length"]):
+            raise errors.MantaError("content-length mismatch: expected %d, "
+                "got %s" % (int(res["content-length"]), content))
+
+        if res.get("content-md5"):
+            md5 = hashlib.md5(content)
+            content_md5 = base64.b64encode(md5.digest())
+            if content_md5 != res["content-md5"]:
+                raise errors.MantaError("content-md5 mismatch: expected %s, "
+                    "got %s" % (res["content-md5"], content_md5))
+
+        return content
+
     def get_size(self, path):
         mpath = self.create_manta_path(path)
-        res, content = self.client._request(path=mpath, method='HEAD')
+        headers = {
+            'cache-control': 'no-cache'
+        }
+        res, content = self.client._request(path=mpath, method='HEAD', headers=headers)
         return int(res["content-length"])
 
     def stream_write(self, path, fp):
@@ -142,7 +169,7 @@ class Storage(driver.Base):
 
     def stream_read(self, path, bytes_range=None):
         mpath = self.create_manta_path(path)
-        return self.client.get_object(mpath)
+        return self.get_object(mpath)
 
     def list_directory(self, path):
         mpath = self.create_manta_path(path)
